#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import subprocess

__cache__ = set()

def ensure_packages(*packages, upgrade=False, quiet=True):
    """
    安装多个 pip 包。包名以位置参数传入，其它选项必须使用关键字参数。

    参数：
        *packages: 位置参数，例如 'requests>=2.0'
        upgrade (keyword-only): 是否使用 --upgrade
        quiet (keyword-only): 是否静默安装（-q）
    """
    if not packages:
        return True

    packages = list(set(packages) - __cache__)
    if not packages:
        return True
    
    cmd = [sys.executable, "-m", "pip", "install"]
    if upgrade:
        cmd.append("--upgrade")
    if quiet:
        cmd.append("-q")
    cmd.extend(packages)

    try:
        subprocess.check_call(cmd)
        __cache__.update(packages)
        return True
    except subprocess.CalledProcessError:
        #TODO: use logger
        pass
        #print("依赖安装失败:", " ".join(packages))
    
    return False


def ensure_requirements(path="requirements.txt", **kwargs):
    with open(path) as f:
        reqs = [line.strip() for line in f if line.strip() and not line.startswith("#")]
    return ensure_requirements(reqs, **kwargs)


if __name__ == "__main__":
    ensure_packages("requests>=2.25", "openai>=1.68.2", "pandas>=2.2.3")
    ensure_packages("requests>=2.25", "openai>=1.68.2", "pandas>=2.2.3")



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import json
from io import BytesIO

import requests

class NoopDiagnose:
    def __getattr__(self, name):
        def noop(*args, **kwargs):
            pass
        return noop

class Diagnose:
    def __init__(self, api_url, api_key):
        self._api_url = api_url
        self._api_key = api_key

    @classmethod
    def create(cls, settings):
        config = settings.get('diagnose')
        if config:
            api_key = config.get('api_key')
            api_url = config.get('api_url')
            enabled = config.get('enabled', True)
            if not api_key or not api_url:
                enabled = False
        else:
            enabled = False

        return cls(api_url, api_key) if enabled else NoopDiagnose()
    
    def report_data(self, data, filename):
        try:
            data = json.dumps(data, ensure_ascii=False, indent=4)
            data = BytesIO(data.encode('utf-8'))
        except Exception as e:
            return False
        
        headers = {'API-KEY': self._api_key}
        files = {'file': (filename, data)}

        try:
            response = requests.post(self._api_url, files=files, headers=headers)
            success = 200 <= response.status_code < 300
        except Exception as e:
            success = False

        return success

    def report_code_error(self, history):
        # Report code execution errors from history
        # Each history entry contains code and execution result
        # We only collect entries with traceback information
        # Returns True if report was sent successfully
        data = []

        for h in history:
            result = h.get('result')
            if not result:
                continue
            traceback = result.get('traceback')
            if not traceback:
                continue
            data.append({
                'code': h.get('code'),
                'traceback': traceback,
                'error': result.get('errstr')
            })

        if data:
            return self.report_data(data, 'code_error.json')
        return True

if __name__ == '__main__':
    settings = {
        'diagnose': {
            'api_key': 'sk-aipy-',
            'api_url': 'https://aipy.ror.workers.dev/',
        }
    }
    diagnose = Diagnose.create(settings)
    diagnose.report_code_error([
        {'code': 'print("Hello, World!")', 'result': {'traceback': 'Traceback (most recent call last):\n  File "test.py", line 1, in <module>\n    print("Hello, World!")\nNameError: name \'print\' is not defined\n', 'errstr': 'NameError: name \'print\' is not defined'}}
    ])



#!/usr/bin/env python
# coding: utf-8

import os
import sys

class Logger:
    def __init__(self, file_path=os.devnull):
        self.terminal = sys.stdout
        self.log = open(file_path, "a", encoding="utf-8")

    def write(self, message):
        self.terminal.write(message)
        self.log.write(message)

    def flush(self):
        self.terminal.flush()
        self.log.flush()

def parse_args():
    import argparse
    from .aipy.config import CONFIG_DIR
    config_help_message = (
        f"Specify the configuration directory.\nDefaults to {CONFIG_DIR} if not provided."
    )

    parser = argparse.ArgumentParser(description="Python use - AIPython", formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument("-c", '--config-dir', type=str, help=config_help_message)
    parser.add_argument('-p', '--python', default=False, action='store_true', help="Python mode")
    parser.add_argument('-g', '--gui', default=False, action='store_true', help="GUI mode")
    parser.add_argument('--debug', default=False, action='store_true', help="Debug mode")
    parser.add_argument('-f', '--fetch-config', default=False, action='store_true', help="login to trustoken and fetch token config")
    parser.add_argument('cmd', nargs='?', default=None, help="Task to execute, e.g. 'Who are you?'")
    return parser.parse_args()

def ensure_wxpython():
    try:
        import wx
    except:
        import subprocess

        cp = subprocess.run([sys.executable, "-m", "pip", "install", 'wxpython'])
        assert cp.returncode == 0

def mainw():
    sys.stdout = Logger()
    sys.stderr = Logger()
    args = parse_args()
    ensure_wxpython()
    from .wxgui import main as aipy_main
    aipy_main(args)

def main():
    args = parse_args()
    if args.python:
        from .main import main as aipy_main
    elif args.gui:
        ensure_wxpython()
        from .wxgui import main as aipy_main
    else:
        from .saas import main as aipy_main
    aipy_main(args)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from pathlib import Path

from .i18n import T
from .task import Task
from .llm import LLM
from .runner import Runner
from .plugin import PluginManager
from .prompt import SYSTEM_PROMPT
from .diagnose import Diagnose

class TaskManager:
    def __init__(self, settings, console):
        self.settings = settings
        self.console = console
        self.task = None
        self.envs = {}
        self.config_files = settings._loaded_files
        self.system_prompt = f"{settings.system_prompt}\n{SYSTEM_PROMPT}"
        plugin_dir = settings.get('plugin_dir') or Path.cwd() / 'plugins'
        self.plugin_manager = PluginManager(plugin_dir)
        self.plugin_manager.load_plugins()
        if settings.workdir:
            workdir = Path.cwd() / settings.workdir
            workdir.mkdir(parents=True, exist_ok=True)
            os.chdir(workdir)
            self._cwd = workdir
        else:
            self._cwd = Path.cwd()
        self._init_environ()
        self._init_api()
        self.diagnose = Diagnose.create(settings)
        self.runner = Runner(settings, console, envs=self.envs)
        self.llm = LLM(settings, console, system_prompt=self.system_prompt)

    @property
    def workdir(self):
        return str(self._cwd)

    @property
    def busy(self):
        return self.task is not None

    def use(self, name):
        ret = self.llm.use(name)
        self.console.print('[green]Ok[/green]' if ret else '[red]Error[/red]')
        return ret

    def done(self):
        if not self.task:
            return
        
        self.diagnose.report_code_error(self.runner.history)
        self.task.done()
        self.task = None

    def save(self, path):
        if self.task:  
            self.task.save(path)

    def _init_environ(self):
        envs = self.settings.get('environ', {})
        for name, value in envs.items():
            os.environ[name] = value

    def _init_api(self):
        api = self.settings.get('api')
        if not api:
            return
        lines = [self.system_prompt]
        for api_name, api_conf in api.items():
            lines.append(f"## {api_name} API")
            desc = api_conf.get('desc')
            if desc: 
                lines.append(f"### API {T('description')}\n{desc}")

            envs = api_conf.get('env')
            if not envs:
                continue

            lines.append(f"### {T('env_description')}")
            for name, (value, desc) in envs.items():
                value = value.strip()
                if not value:
                    continue
                lines.append(f"- {name}: {desc}")
                self.envs[name] = (value, desc)

        self.system_prompt = "\n".join(lines)

    def new_task(self, instruction, llm=None, max_rounds=None, system_prompt=None):
        if llm and not self.llm.use(llm):
            return None
        
        system_prompt = system_prompt or self.system_prompt
        max_rounds = max_rounds or self.settings.get('max_rounds')
        task = Task(instruction, system_prompt=system_prompt, max_rounds=max_rounds)
        task.console = self.console
        task.llm = self.llm
        task.runner = self.runner
        self.task = task
        return task
    
    def __call__(self, instruction, llm=None, max_rounds=None, system_prompt=None):
        if self.task:
            self.task.run(instruction=instruction, llm=llm, max_rounds=max_rounds)
        else:
            task = self.new_task(instruction, llm=llm, max_rounds=max_rounds, system_prompt=system_prompt)
            self.task = task
            task.run()



from .taskmgr import TaskManager
from .runner import Runner
from .llm import LLM, ChatHistory
from .i18n import T, set_lang
from .plugin import event_bus

__all__ = ['TaskManager', 'Runner', 'LLM', 'ChatHistory', 'T', 'set_lang', 'event_bus']




CONSOLE_HTML_FORMAT = """
<!DOCTYPE html>
<html>
<head>
<meta charset="UTF-8">
<style>
.r1 {{color: #008080; text-decoration-color: #008080; font-weight: bold}}
.r2 {{color: #800000; text-decoration-color: #800000; font-weight: bold}}
.r3 {{color: #008000; text-decoration-color: #008000}}
.r4 {{color: #008000; text-decoration-color: #008000; font-weight: bold}}
.r5 {{color: #800080; text-decoration-color: #800080}}
.r6 {{color: #ff00ff; text-decoration-color: #ff00ff}}
.r7 {{color: #008080; text-decoration-color: #008080}}
.r8 {{color: #808000; text-decoration-color: #808000}}
.r9 {{color: #800000; text-decoration-color: #800000}}
.r10 {{color: #008080; text-decoration-color: #008080; background-color: #ffffff; font-weight: bold}}
.r11 {{background-color: #ffffff}}
.r12 {{color: #444444; text-decoration-color: #444444; background-color: #ffffff}}
.r13 {{color: #cc3366; text-decoration-color: #cc3366; background-color: #ffffff}}
.r14 {{color: #222222; text-decoration-color: #222222; background-color: #ffffff}}
.r15 {{color: #006699; text-decoration-color: #006699; background-color: #ffffff}}
.r16 {{color: #228822; text-decoration-color: #228822; background-color: #ffffff}}
.r17 {{color: #aa9900; text-decoration-color: #aa9900; background-color: #ffffff}}
.r18 {{color: #6633cc; text-decoration-color: #6633cc; background-color: #ffffff}}
.r19 {{color: #808000; text-decoration-color: #808000; font-weight: bold}}
.r20 {{color: #333333; text-decoration-color: #333333; background-color: #ffffff; font-weight: bold}}
.r21 {{color: #999999; text-decoration-color: #999999; background-color: #ffffff}}
.r22 {{color: #222222; text-decoration-color: #222222}}
.r23 {{font-weight: bold}}
body {{
    color: #000000;
    background-color: #ffffff;
}}
</style>
</head>
<body>
    <pre style="font-family:Menlo,'DejaVu Sans Mono',consolas,'Courier New',monospace"><code style="font-family:inherit">{code}</code></pre>
</body>
</html>
"""

DISCLAIMER_TEXT = """[bold yellow]⚠️  风险提示与免责声明 ⚠️[/bold yellow]

本程序会执行由大型语言模型（LLM）自动生成的代码。请您在继续使用前，务必阅读并理解以下内容：

[bold red]1. 风险提示：[/bold red]
- 自动生成的代码可能包含逻辑错误、性能问题或不安全操作（如删除文件、访问网络、执行系统命令等）。
- 本程序无法保证生成代码的准确性、完整性或适用性。
- 在未充分审查的情况下运行生成代码，可能会对您的系统、数据或隐私造成损害。

[bold yellow]2. 免责声明：[/bold yellow]
- 本程序仅作为开发与测试用途提供，不对由其生成或执行的任何代码行为承担责任。
- 使用本程序即表示您理解并接受所有潜在风险，并同意对因使用本程序产生的任何后果自行负责。"""



class Plugin:
    """Trustoken all-in-one API plugin for AIPY
    """
    def on_exec(self, blocks):
        """
        执行代码事件
        param: blocks
        """
        blocks['main'] = blocks['main'].replace('https://restapi.amap.com/', 'https://api.trustoken.cn/aio-api/amap/')